#!/bin/sh
#
# Script to be run after a firmware update
#
# This is used for performing tasks such as renaming CDS 
# params/attributes that have changed names, or other such post-
# processing that we really only need to do once, and which we
# don't need the new firmware to do.
#

# Moves an attribute from one path to another (persistently)
# Usage: move_attr <from> <to> <type>
move_persist_attr() {
	val=`cdsutil get "$1" 2> /dev/null`
	if [ "$?" = "0" ]; then
		cdsutil set -p "$2" $val $3 2> /dev/null
		cdsutil clear "$1" 2> /dev/null
	fi
}


# Replaces the .AlarmOnTrue attribute with .AlarmOnValue
# Since the only place we had let the user configure alarm limits
# was in /99 (EMP), that's the only place we need to look for this
replace_emp_alarmontrue() {
	move_persist_attr "/99/mRC1Status/.AlarmOnTrue" "/99/mRC1Status/.AlarmOnValue" bool
	move_persist_attr "/99/mRC1Status/.AlarmOnTrue" "/99/mRC1Status/.AlarmOnValue" bool
}


# Removes any persistently-set UPnP options (never should have been persisted)
remove_persist_upnp_options() {
	BASE="/Software/Data Servers/UPnP Server"
	cdsutil remove "$BASE/XMLFileName" 2> /dev/null
	cdsutil remove "$BASE/XMLDescPath" 2> /dev/null
	cdsutil remove "$BASE/URLPath" 2> /dev/null
}


# The web pages previously saved the date/time persistently
remove_persist_date_time() {
	cdsutil remove "/Software/System Settings/Date Time" 2> /dev/null
}


OPENSSL_CONF=/etc/openssl.cnf
export OPENSSL_CONF

# Check to see whether we have a CA which was created with a bogus start/end
# date due to manufacturing issues.
check_ca_signing_date() {
	if [ -f "/etc/rc.platform" ]; then
		. /etc/rc.platform
		SSL_DIR="${CFG_SSL_DIR}"
	else
		SSL_DIR="/jffs/ssl"
	fi

	if [ -f "${SSL_DIR}/cacert.pem" ]; then
		date=`openssl x509 -in ${SSL_DIR}/cacert.pem -noout -startdate | cut -d"=" -f2`
		if [ "$?" -ne 0 ]; then
			return
		fi

		year=`date -d "${date}" -D "%b %e %T %Y %Z" "+%Y"`
		if [ "$?" -ne 0 ]; then
			return
		fi

		if [ -n "${year}" -a "${year}" -lt 2000 ]; then
			# Certificate was generated prior to 2000, which
			# shouldn't happen - this is probably a bug, so
			# remove the SSL cert/CA and let it be re-generated
			# later.
			rm -rf "${SSL_DIR}"
		fi
	fi
}


# If updating from a device which didn't have zeroconf settings, default to
# zeroconf being disabled when DHCP is also disabled - this ensures forward
# compatibility for statically configured devices.
set_zeroconf() {
	local nifbase="/Software/System Settings/Network/Interface"
	local children=`cdsutil get "${nifbase}/Children" 2> /dev/null`
	local OLD_IFS="${IFS}"
	IFS=";"
	for ifname in ${children} ; do
		cdsutil get "${nifbase}/${ifname}/Zeroconf Enabled/Value" > /dev/null 2>&1
		if [ "$?" -ne 0 ]; then
			local dhcp_on=`cdsutil get "${nifbase}/${ifname}/DHCP Enabled/Value" 2> /dev/null`
			if [ "$?" -eq 0 -a -n "${dhcp_on}" ]; then
				# If DHCP disabled, disable zeroconf also
				cdsutil set -p "${nifbase}/${ifname}/Zeroconf Enabled/Value" "${zen}" bool
			fi
		fi
	done
	IFS="${OLD_IFS}"
}


# If upgrading a PXG which stored its waveforms in /flash/wav, migrate them to
# "/flash/export/waveforms" instead.
migrate_waveform_dir() {
	local old_wav_dir="/flash/wav"
	local new_wav_dir="/flash/export/waveforms"
	
	if [ -d "${old_wav_dir}" -a ! -d "${new_wav_dir}" ]; then
		mkdir -p "`dirname ${new_wav_dir}`"
		mv "${old_wav_dir}" "${new_wav_dir}"
	fi
}


# Get the current firmware version & revision
[ -f "/etc/firmware_version" ] && . /etc/firmware_version
[ -z "$HWREF_VERSION" ] && HWREF_VERSION=0

[ -f "/etc/firmware_rev" ] && HWREF_REV=`cat /etc/firmware_rev`
[ -z "$HWREF_REV" ] && HWREF_REV=0


# For now we call these functions unconditionally, since we don't know what
# the user's past upgrade path may have been.  In the future we can make more
# assumptions here to avoid running needless tests.
replace_emp_alarmontrue
remove_persist_upnp_options
remove_persist_date_time
check_ca_signing_date
set_zeroconf
migrate_waveform_dir

exit 0

